package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV566 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv566:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private RankStrategy4RegionMergeModelV563 modelV563Service;

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);

        List<Video> result = modelV563Service.mergeAndRankRovRecall(param);
        if (null != result && !result.isEmpty()) {
            try {
                Integer versionCode = param.getVersionCode();
                int orderVersionCode = mergeWeight.getOrDefault("orderVersionCode", 1500D).intValue();
                if (null != versionCode && versionCode == orderVersionCode) {
                    int keepTopN = mergeWeight.getOrDefault("keepTopN", 1D).intValue();
                    int secondN = mergeWeight.getOrDefault("secondN", 4D).intValue();
                    boolean secondFirstFlag = mergeWeight.getOrDefault("secondFirstFlag", 1D).intValue() > 0;
                    return resort(keepTopN, secondN, secondFirstFlag, result);
                }
            } catch (Exception e) {
                log.error("diversity resort error", e);
            }
        }
        return result;
    }

    private List<Video> resort(int keepTopN, int secondN, boolean secondFirstFlag, List<Video> videos) {
        // split
        List<Video> result = new ArrayList<>();
        List<Video> list2nd = new ArrayList<>();
        List<Video> list3rd = new ArrayList<>();
        for (int i = 0; i < videos.size(); i++) {
            if (i < keepTopN) {
                result.add(videos.get(i));
            } else if (i < secondN) {
                list2nd.add(videos.get(i));
            } else {
                list3rd.add(videos.get(i));
            }
        }

        // merge
        if (secondFirstFlag) {
            fillVideo(list2nd, list3rd, result);
        } else {
            fillVideo(list3rd, list2nd, result);
        }
        return result;
    }

    private void fillVideo(List<Video> list1, List<Video> list2, List<Video> result) {
        int index = 0;
        for (; index < list1.size() && index < list2.size(); index++) {
            result.add(list1.get(index));
            result.add(list2.get(index));
        }
        for (; index < list1.size(); index++) {
            result.add(list1.get(index));
        }
        for (; index < list2.size(); index++) {
            result.add(list2.get(index));
        }
    }
}
